
<a href="http://depsy.org/package/python/matplotlib">
  <img src="{{ pathto('_static/depsy_badge.svg', 1) }}">
</a>

<br/>

Travis-CI: <a href="https://travis-ci.org/matplotlib/matplotlib">
  <img src="https://travis-ci.org/matplotlib/matplotlib.svg?branch=master"/>
</a>
<br/>
