.. Matplotlib
.. _`Matplotlib`: http://matplotlib.org
.. _`Matplotlib github`: https://github.com/matplotlib/matplotlib

.. _`Matplotlib mailing list`: https://mail.python.org/mailman/listinfo/matplotlib-devel
